/*AxisSense.java
 *@author Iván Gordón
 *@author Alejandro Merello
 *NOTA: Antes de compilar es necesario ingresar la información de cuenta de correo (Línea 197+)
 */

import java.awt.*;              //Graphics2D, Dimension, Graphics, Image, BorderLayout, Color
import javax.swing.*;           //JComponent, JPanel, JFrame, JLabel, ImageIcon, BorderFactory
import java.io.*;               //BufferedReader, PrintWriter, InputStreamReader, File, IOException
import javax.mail.*;            //Session, Transport, RecipientType
import javax.mail.internet.*;   //MimeMessage, MimeBodyPart, MimeMultipart
import java.awt.image.BufferedImage;
import javax.imageio.ImageIO;
import java.net.URL;
import java.net.MalformedURLException;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Calendar;
import java.net.InetAddress;
import java.util.Properties;

public class AxisSense extends JComponent {
  AxisSensePanel panel;
  Timer timer;
  Runtime runTime;
  Process process;
  BufferedReader in;
  PrintWriter out;
  int i; long time;
  String ip,email;

  //Constructor de AxisSense
  public AxisSense() {
    panel = new AxisSensePanel();                                       //Panel principal de la aplicación, muestra el video.
    timer = new Timer();
    JFrame.setDefaultLookAndFeelDecorated(true);
    JFrame frame = new JFrame("Axis Sense");

    //////////////////////////////////////////////////
    //Ventana emergente con configuración           //
    //////////////////////////////////////////////////
    JPanel panelVar = new JPanel();
    panelVar.setLayout(new BoxLayout(panelVar, BoxLayout.PAGE_AXIS));
    JLabel labelIp = new JLabel("Ingrese dirección IP de la cámara");
    JTextField ipField = new JTextField("192.168.2.110");
    JLabel labelEmail = new JLabel("Ingrese email donde enviar alertas");
    JTextField emailField = new JTextField("mgelo330@gmail.com");
    panelVar.add(labelIp);
    panelVar.add(ipField);
    panelVar.add(labelEmail);
    panelVar.add(emailField);
    
    boolean check;                                                      //Booleano para saber si hay que quedarse o avanzar
    do{                                                                 
        check=false;
        JOptionPane.showMessageDialog(frame,panelVar);
        ip = ipField.getText();
        String[] ips = ip.split("\\.");
        if(ips.length!=4) check=true;                                   //Este bloque if-else comprueba que sea una dirección IP válida
        else {
            for(int num=0,i=0;(i<4) && !check;i++){
                try{
                    num=Integer.parseInt(ips[i]);
                } catch(NumberFormatException e){check = true;}
                if(check || (num<0 || num>255)) check = true;
            }
        }
        if(!check) {                                                    //Este bloque if-else comprueba que sea un email válido
            email = emailField.getText();
            ips = email.split("@");
            if(ips.length!=2) check=true;
            else {
                ips=ips[1].split("\\.");
                if(ips.length<2) check=true;
            }
        }
    }while(check);                                                      //De lo contrario el panel nunca termina.

    //////////////////////////////////////////////////
    //Abre Matlab                                   //
    //////////////////////////////////////////////////
    try {                                                                       // Este try con su respectivo catch se encargan de ejecutar el proceso 
      runTime = Runtime.getRuntime();                                           //matlab y anclar su entrada y salida estándar a in y out respectivamente.
      process = runTime.exec("matlab -nodesktop");
      in = new BufferedReader(new InputStreamReader(process.getInputStream()));
      out = new PrintWriter(process.getOutputStream(),true);
    } catch (IOException e) {
      System.out.println("exec failed");
      System.exit(-1);
    }

    //////////////////////////////////////////////////
    //Instancia ventana del video y evento periódico//
    //////////////////////////////////////////////////
    JLabel label = new JLabel("Medición remota",JLabel.CENTER);
    frame.getContentPane().add(label,BorderLayout.NORTH);
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.add(panel);
    frame.pack();
    JTextField textField = new JTextField(20);
    frame.setVisible(true);
    time = 0;
    timer.scheduleAtFixedRate(new ImageUpdate(ip,email), 0, 2000);
  }
  
  //Método main sólo construye una instancia de si mismo. 
  public static void main(String[] args) throws Exception {
    new AxisSense();
  }

  //Método engargado de hacer la tarea periódica de actualización. Descarga una imagen, renombra la imagen antigua
  //y ejecuta el script de matlab "detect" el cual escribe ALARM ante una detección de movimiento. En cuyo caso ImageUpdate
  //envía un Email con la información del evento al correo predeterminado.
  class ImageUpdate extends TimerTask {
    Image image = null;
    String ip,email;
    ImageUpdate(String ipSrc,String emailSrc) {
        ip=ipSrc;
        email=emailSrc;
    }
    public void run() {
      try {
        URL url = new URL("http://"+ip+"/axis-cgi/jpg/image.cgi?resolution=320x240");
        image = new ImageIcon(url).getImage();
      } catch (MalformedURLException e) {} 
        catch (IOException e) {}
      BufferedImage bimg = new BufferedImage(image.getWidth(null), image.getHeight(null), BufferedImage.TYPE_INT_RGB);
      Graphics2D ge = bimg.createGraphics(); ge.drawImage(image, 0, 0, null);

      try {
        File newIm = new File("2.jpg");
	    File oldIm = new File("1.jpg");
	    newIm.renameTo(oldIm);
        File outputfile = new File("2.jpg");
        ImageIO.write(bimg, "jpg", outputfile);
      } catch (IOException e) {}
      
      out.println("detect");
      try {
        while(in.ready()) {
               if(i==0 && in.read()=='A')i=1;
          else if(i==1 && in.read()=='L')i=2;
          else if(i==2 && in.read()=='A')i=3;
          else if(i==3 && in.read()=='R')i=4;
          else if(i==4 && in.read()=='M')i=5;
          else i=0;
          if(i==5) { 
            i=0;
            Calendar now = Calendar.getInstance();
            if((now.getTimeInMillis() - time)>60*1000) {    //Sólo acepta eventos cada 1 minuto para no inundar el Inbox
              System.out.println("ALARM");
              File newIm = new File("1.jpg");
	          File oldIm = new File("3.jpg");
	          newIm.renameTo(oldIm);
              SmtpSend sendmail = new SmtpSend(email);
              sendmail.start();
              time = now.getTimeInMillis();}
          }
        }
      } catch (IOException e) {}
      panel.setImage(image);
      panel.repaint();
    }
  }
}

//Clase engargada de dibujar la GUI
class AxisSensePanel extends JPanel {
  Image image;
  public AxisSensePanel() {
    setBorder(BorderFactory.createLineBorder(Color.black));
    image = null;
  }

  public Dimension getPreferredSize() {
    return new Dimension(400,400);
  }

  public void paintComponent(Graphics g) {
    super.paintComponent(g);       
    if(image!=null) {
      g.drawImage(image, 200-image.getWidth(null)/2, 200-image.getHeight(null)/2, this);
      image.flush();
    }
    g.finalize();
  }
  public void setImage(Image img) {
    image = img;
  }
}

//Clase que extiende a Thread encargada de enviar el email con la foto.
class SmtpSend extends Thread {
  String host, username, password, to;

  SmtpSend() {
    host = /* INGRESAR SERVIDOR SMTP (ej. smtp.google.com) */;
    username = /* INGRESAR CUENTA GMAIL FUENTE (ej. google@gmail.com) */;
    password = /* INGRESAR PASSWORD EN TEXTO PLANO */;
    to = /* INGRESAR EMAIL DESTINO*/;
  }

  SmtpSend(String email) {
    host = /* INGRESAR SERVIDOR SMTP (ej. smtp.google.com) */;
    username = /* INGRESAR CUENTA GMAIL FUENTE (ej. google@gmail.com) */;
    password = /* INGRESAR PASSWORD EN TEXTO PLANO */;
    to = /* INGRESAR EMAIL DESTINO*/;
  }

  public void run () {
    Properties props = new Properties();
    props.put("mail.smtps.auth", "true");
    props.put("mail.smtps.host", host);
    Session session = Session.getInstance(props, null);
    MimeMessage msg = new MimeMessage(session);
    try {
      msg.setFrom(new InternetAddress(username));
      msg.setRecipients(Message.RecipientType.TO, InternetAddress.parse(to, false));
      msg.setSubject("[AxisSense] Evento detectado!");
      MimeBodyPart mbp1 = new MimeBodyPart();
      MimeBodyPart mbp2 = new MimeBodyPart();
      MimeMultipart mp = new MimeMultipart();
      mbp1.setText("Se ha detectado movimiento en la camara IP, se adjunta imagen del momento.");
      mbp2.attachFile("3.jpg");
      mp.addBodyPart(mbp1);
      mp.addBodyPart(mbp2);
      msg.setContent(mp);
      Transport t = session.getTransport("smtps");
      try {
        t.connect(host, username, password);
        t.sendMessage(msg, msg.getAllRecipients());
      } finally {
        t.close();
      }
    } catch(Exception e){}
  }
}
